@@ -2,11 +2,15 @@ package Archive::Ar::Libarchive;
 
 use strict;
 use warnings;
-use Carp qw( carp );
-use File::Basename qw( basename );
+use base qw( Exporter );
+use constant COMMON => 1;
+use constant BSD    => 2;
+use constant GNU    => 3;
+use Carp qw( carp longmess );
+use File::Basename ();
 
 # ABSTRACT: Interface for manipulating ar archives with libarchive
-our $VERSION = '0.12'; # VERSION
+our $VERSION = '2.00'; # VERSION
 
 unless($^O eq 'MSWin32')
 {
@@ -17,6 +21,8 @@ unless($^O eq 'MSWin32')
 require XSLoader;
 XSLoader::load('Archive::Ar::Libarchive', $VERSION);
 
+our @EXPORT_OK = qw( COMMON BSD GNU );
+
 
 sub new
 {
@@ -28,7 +34,7 @@ sub new
   {
     unless($self->read($filename_or_handle))
     {
-      return $self->_warn("new() failed on filename for filehandle read");
+      return $self->_error("new() failed on filename for filehandle read");
     }
   }
   
@@ -42,8 +48,9 @@ sub read
 
   my $ret = 0;
   
-  if(ref $filename_or_handle eq 'GLOB')
+  if(ref $filename_or_handle)
   {
+    return $self->_error("Not a filehandle") unless eval{*$filename_or_handle{IO}} or $filename_or_handle->isa('IO::Handle');
     my $buffer;
     $ret = $self->_read_from_callback(sub {
       my $br = read $filename_or_handle, $buffer, 1024;
@@ -72,6 +79,32 @@ sub read_memory
 }
 
 
+sub chmod
+{
+  my($self, $filename, $mode) = @_;
+  $self->_chmod($filename, $mode + 0 eq $mode ? $mode : oct($mode));
+}
+
+
+sub chown
+{
+  my($self, $filename, $uid, $gid) = @_;
+  $self->_chown($filename, $uid, $gid);
+}
+
+
+sub remove
+{
+  my $self = shift;
+  my $count = 0;
+  foreach my $pathname (@{ ref $_[0] ? $_[0] : \@_ })
+  {
+    $count += $self->_remove($pathname);
+  }
+  $count;
+}
+
+
 sub list_files
 {
   my $list = shift->_list_files;
@@ -87,18 +120,18 @@ sub add_files
   {
     unless(-r $filename)
     {
-      $self->_warn("No such file: $filename");
+      $self->_error("No such file: $filename");
       next;
     }
     my @props = stat($filename);
     unless(@props)
     {
-      $self->_warn("Could not stat $filename.");
+      $self->_error("Could not stat $filename.");
       next;
     }
     
     open(my $fh, '<', $filename) || do {
-      $self->_warn("Unable to open $filename $!");
+      $self->_error("Unable to open $filename $!");
       next;
     };
     binmode $fh;
@@ -108,7 +141,7 @@ sub add_files
     my $data = do { local $/; <$fh> };
     close $fh;
     
-    $self->add_data(basename($filename), $data, {
+    $self->add_data(File::Basename::basename($filename), $data, {
       date => $props[9],
       uid  => $props[4],
       gid  => $props[5],
@@ -138,62 +171,78 @@ sub write
   my($self, $filename) = @_;
   if(defined $filename)
   {
-    my $status = $self->_write_to_filename($filename);
-    return unless $status;
-    return $status;
+    my $fh;
+  
+    if(ref $filename)
+    {
+      return $self->_error("Not a filehandle") unless eval{*$filename{IO}} or $filename->isa('IO::Handle');
+      $fh = $filename;
+      
+      return $self->_write_to_callback(sub {
+        my($archive, $buffer) = @_;
+        print $fh $buffer;
+        length $buffer;
+      });
+    }
+
+    return $self->_write_to_filename($filename);
   }
   else
   {
-    #my $content = '';
-    ## TODO: doesn't work
-    #my $status = $self->_write_to_callback(sub {
-    #  my($archive, $buffer) = @_;
-    #  $content .= $buffer;
-    #  length $buffer;
-    #});
-    
-    use File::Temp qw( tempdir );
-    use File::Spec;
-    my $dir = tempdir( CLEANUP => 1 );
-    my $fn = File::Spec->catfile($dir, 'archive.ar');
-    my $status = $self->_write_to_filename($fn);
+    my $content = '';
+    my $status = $self->_write_to_callback(sub {
+      my($archive, $buffer) = @_;
+      $content .= $buffer;
+      length $buffer;
+    });
     return unless $status;
-    
-    open my $fh, '<', $fn;
-    binmode $fh;
-    my $data = do { local $/; <$fh> };
-    close $fh;
-    unlink $fn;
-    
-    return $data;
+    return $content;
   }
 }
 
 
-sub remove
+sub get_handle
 {
-  my $self = shift;
-  my $count = 0;
-  foreach my $pathname (@{ ref $_[0] ? $_[0] : \@_ })
-  {
-    $count += $self->_remove($pathname);
-  }
-  $count;
+  my $data = shift->get_data(@_);
+  return unless defined $data;
+  open my $fh, '<', \$data;
+  $fh;
 }
 
 
+sub set_output_format_bsd
+{
+  carp "set_output_format_bsd is deprecated, use \$ar->set_opt(type => BSD) instead";
+  shift->set_opt(type => BSD);
+}
+
+sub set_output_format_svr4
+{
+  carp "set_output_format_bsd is deprecated, use \$ar->set_opt(type => COMMON) instead";
+  shift->set_opt(type => COMMON);
+}
+
 sub DEBUG
 {
+  carp "DEBUG is deprecated, use \$ar->set_opt(\"warn\", 1) instead";
   my($self, $value) = @_;
-  $value = 1 unless defined $value;
-  $self->_set_debug($value);
-  return;
+  $self->set_opt(warn => 1) unless defined $value and $value == 0;
 }
 
-sub _warn
+sub _error
 {
-  my($self, $warning) = @_;
-  carp $warning if $self->_get_debug;
+  my($self, $message) = @_;
+  my $opt_warn = $self->get_opt('warn');
+  my $longmess = longmess $message;
+  $self->_set_error($message, $longmess);
+  if($opt_warn > 1)
+  {
+    carp $longmess;
+  }
+  elsif($opt_warn)
+  {
+    carp $message;
+  }
   return;
 }
 
@@ -211,7 +260,7 @@ Archive::Ar::Libarchive - Interface for manipulating ar archives with libarchive
 
 =head1 VERSION
 
-version 0.12
+version 2.00
 
 =head1 SYNOPSIS
 
@@ -248,9 +297,8 @@ The interface attempts to be identical (with a couple of minor
 extensions) to L<Archive::Ar> and the documentation presented here is 
 based on that module. The diagnostic messages issued on error mostly 
 come directly from libarchive, so they will likely not match exactly 
-what L<Archive::Ar> would produce, but it should issue a warning (when 
-L<Archive::Ar::Libarchive#DEBUG> is turned on) under similar 
-circumstances.
+what L<Archive::Ar> would produce, but it should issue a warning under
+similar  circumstances.
 
 The main advantage of L<Archive::Ar> over this module is that it is 
 written in pure perl, and thus does not require a compiler or 
@@ -259,23 +307,88 @@ is that it supports GNU (read) and BSD (read and write) extensions for
 longer member filenames.  As an XS module using libarchive it may also
 be faster.
 
+You may notice that the API to L<Archive::Ar::Libarchive> and
+L<Archive::Ar> is similar to L<Archive::Tar> and this was done
+intentionally to keep similarity between the Archive::* modules.
+
 =head1 METHODS
 
 =head2 new
 
  my $ar = Archive::Ar::Libarchive->new;
  my $ar = Archive::Ar::Libarchive->new($filename);
- my $ar = Archive::Ar::Libarchive->new($fh, $debug);
+ my $ar = Archive::Ar::Libarchive->new($fh);
 
 Returns a new L<Archive::AR::Libarchive> object.  Without a filename or 
 glob, it returns an empty object.  If passed a filename as a scalar or a 
 GLOB, it will attempt to populate from either of those sources.  If it 
-fails, you will receive undef, instead of an object reference.
+fails, you will receive C<undef>, instead of an object reference.
 
-This also can take a second optional debugging parameter.  This acts 
-exactly as if L<Archive::Ar::Libarchive#DEBUG> is called on the object 
-before it is returned.  If you have a L<Archive::Ar::Libarchive#new> 
-that keeps failing, this should help.
+=head2 set_opt
+
+ $ar->set_opt($name, $value);
+
+Assign option C<$name> value C<$value>.  Supported options include:
+
+=over 4
+
+=item warn
+
+Warning level.  Levels are zero for no warnings, 1 for brief warnings,
+and 2 for warnings with a stack trace.  Default is zero.
+
+Warnings that originate with libarchive itself will not include a
+stacktrace, even with a warn level set to 2.
+
+=item chmod
+
+Change the file permissions of files created when extracting.  Default
+is true (non-zero).
+
+This option is provided only for compatibility with L<Archive::Ar>.
+Libarchive does not provide an equivalent to this option, so setting
+it to false will has no effect.
+
+=item same_perms
+
+When setting file permissions, use the values in the archive unchanged.
+If false, removes setuid bits and applies the user's umask.  Default
+is true.
+
+In L<Archive::Ar> this option is true for root only.
+
+=item chown
+
+Change the owners of extracted files, if possible.  Default is true.
+
+=item type
+
+Archive type.  May be GNU, BSD or COMMON, or undef if no archive
+has been read.  Defaults to the type of the archive read or C<undef>.
+
+=back
+
+=head2 get_opt
+
+ my $value = $ar->get_opt($name);
+
+Returns the value of the option C<$name>.
+
+=head2 type
+
+ my $type = $ar->type;
+
+Returns the type of the ar archive.  The type is undefined until an archive
+is loaded.  If the archive displays characteristics of a GNU-style archive,
+GNU is returned.  If it looks like a bsd-style archive, BSD is returned.
+Otherwise, COMMON is returned.  Note that unless filenames exceed 16
+characters in length, bsd archives look like the common format.
+
+=head2 clear
+
+ $ar->clear;
+
+Clears the current in-memory archive.
 
 =head2 read
 
@@ -283,9 +396,9 @@ that keeps failing, this should help.
  my $br = $ar->read($fh);
 
 This reads a new file into the object, removing any ar archive already
-represented in the object.
-
-Returns the number of bytes read, undef on failure.
+represented in the object.  The argument may be either a filename,
+filehandle or IO::Handle object.  Returns the number of bytes read,
+C<undef> on failure.
 
 =head2 read_memory
 
@@ -295,7 +408,60 @@ This reads information from the first parameter, and attempts to parse
 and treat it like an ar archive. Like L<Archive::Ar::Libarchive#read>, 
 it will wipe out whatever you have in the object and replace it with the 
 contents of the new archive, even if it fails. Returns the number of 
-bytes read (processed) if successful, undef otherwise.
+bytes read (processed) if successful, C<undef> otherwise.
+
+=head2 contains_file
+
+ my $bool = $ar->contains_file($filename)
+
+Returns true if the archive contains a file with the name C<$filename>.
+Returns C<undef> otherwise.
+
+=head2 extract
+
+ $ar->extract;
+
+Extract all files from the archive.  Extracted files are assigned the
+permissions and modification time stored in the archive, and, if possible,
+the user and group ownership.  Returns true on success, C<undef> for failure.
+
+=head2 extract_file
+
+ $ar->extract_file($filename);
+
+Extracts a single file from the archive.  The extracted file is assigned
+the permissions and modification time stored in the archive, and, if
+possible, the user and group ownership.  Returns true on success,
+C<undef> for failure.
+
+=head2 rename
+
+ $ar->rename($filename, $newname);
+
+Changes the name of a file in the in-memory archive.
+
+=head2 chmod
+
+ $ar->chmod($filename, $mode);
+
+Change the permission mode of the member to C<$mode>.
+
+=head2 chown
+
+ $ar->chown($filename, $uid, $gid);
+ $ar->chown($filename, $uid);
+
+Change the ownership of the member to user id C<$udi> and (optionally)
+group id C<$gid>.  Negative id values are ignored.
+
+=head2 remove
+
+ my $count = $ar->remove(@pathnames);
+ my $count = $ar->remove(\@pathnames);
+
+The remove method takes a filenames as a list or as an arrayref, and removes
+them, one at a time, from the Archive::Ar object.  This returns the number
+of files successfully removed from the archive.
 
 =head2 list_files
 
@@ -322,7 +488,7 @@ L<Archive::Ar::Libarchive#add_files> will store the uid, gid, mode,
 size, and creation date of the file as returned by 
 L<stat|perlfunc#stat>.
 
-returns the number of files successfully added, or undef on failure.
+returns the number of files successfully added, or C<undef> on failure.
 
 =head2 add_data
 
@@ -341,7 +507,7 @@ require data on disk to be present. The data is a hash that looks like:
  };
 
 You cannot add_data over another file however.  This returns the file 
-length in bytes if it is successful, undef otherwise.
+length in bytes if it is successful, C<undef> otherwise.
 
 =head2 write
 
@@ -351,7 +517,7 @@ length in bytes if it is successful, undef otherwise.
 This method will return the data as an .ar archive, or will write to the 
 filename present if specified. If given a filename, 
 L<Archive::Ar::Libarchive#write> will return the length of the file 
-written, in bytes, or undef on failure. If the filename already exists, 
+written, in bytes, or C<undef> on failure. If the filename already exists, 
 it will overwrite that file.
 
 =head2 get_content
@@ -360,7 +526,7 @@ it will overwrite that file.
 
 This returns a hash with the file content in it, including the data that the
 file would naturally contain.  If the file does not exist or no filename is
-given, this returns undef. On success, a hash is returned with the following
+given, this returns C<undef>. On success, a hash is returned with the following
 keys:
 
 =over 4
@@ -395,36 +561,28 @@ The contained data
 
 =back
 
-=head2 remove
+=head2 get_data
 
- my $count = $ar->remove(@pathnames);
- my $count = $ar->remove(\@pathnames);
+ my $data = $ar->get_data($filename);
 
-The remove method takes a filenames as a list or as an arrayref, and removes
-them, one at a time, from the Archive::Ar object.  This returns the number
-of files successfully removed from the archive.
+Returns a scalar containing the file data of the given archive member.
+On error returns C<undef>.
 
-=head2 set_output_format_bsd
+=head2 get_handle
 
- $ar->set_output_format_bsd;
+ my $handle = $ar->get_handle($filename);
 
-Sets the output format produced by L<Archive::Ar::Libarchive#write> to 
-use BSD format. Note: this method is not available in L<Archive::Ar>.
+Returns a file handle to the in-memory file data of the given archive
+member.  On error returns C<undef>.  This can be useful for unpacking
+nested archives.
 
-=head2 set_output_format_svr4
+=head2 error
 
- $ar->set_output_format_svr4;
+ my $error_string = $ar->error($trace);
 
-Sets the output format produced by L<Archive::Ar::Libarchive#write> to 
-System VR4 format. Note: this method is not available in L<Archive::Ar>.
-
-=head2 DEBUG
-
- $ar->DEBUG;
- $ar->DEBUG(0);
-
-This method turns on debugging.  To Turn off pass a false value in as 
-the argument.
+Returns the current error string, which is usually the last error
+reported.  If a true value is provided, returns the error message
+and stack trace.
 
 =head1 SEE ALSO
 
